//==========================================================================
//
//      hal/mips/bcm33xx/includebcm33xx_serial.h
//
//      Broadcom 33xx register definitions.
//
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    msieweke
// Contributors: 
// Date:         2003-08-08
// Purpose:      Broadcom 33xx register definitions.
// Description:  Broadcom 33xx register definitions.
//####DESCRIPTIONEND####
//==========================================================================

#ifndef BCM33XX_REGS_H
#define BCM33XX_REGS_H

// ----------------------------------------------------------------------------
// Broadcom 33xx UART registers.  This was copied from bcm_map.h.
// ----------------------------------------------------------------------------

typedef struct Uart
{
    cyg_uint8   unused0;
    cyg_uint8   control;
        #define BRGEN           0x80    /* Control register bit defs */
        #define TXEN            0x40
        #define RXEN            0x20
        #define LOOPBK          0x10
        #define TXPARITYEN      0x08
        #define TXPARITYEVEN    0x04
        #define RXPARITYEN      0x02
        #define RXPARITYEVEN    0x01

    cyg_uint8   config;
        #define XMITBREAK   0x40
        #define BITS5SYM    0x00
        #define BITS6SYM    0x10
        #define BITS7SYM    0x20
        #define BITS8SYM    0x30
        #define ONESTOP     0x07
        #define ONEPTFIVESTOP     0x0b
        #define TWOSTOP     0x0f

    /* 4-LSBS represent STOP bits/char
     * in 1/8 bit-time intervals.  Zero
     * represents 1/8 stop bit interval.
     * Fifteen represents 2 stop bits.
     */
    cyg_uint8   fifoctl;
        #define RSTTXFIFOS  0x80
        #define RSTRXFIFOS  0x40

    /* 5-bit TimeoutCnt is in low bits of this register.
     *  This count represents the number of characters
     *  idle times before setting receive Irq when below threshold
     */
    cyg_uint32  baudword;
    /* When divide SysClk/2/(1+baudword) we should get 32*bit-rate
     */

    cyg_uint8   txf_levl;  /* Read-only fifo depth */
    cyg_uint8   rxf_levl;  /* Read-only fifo depth */
    cyg_uint8   fifocfg; /* Upper 4-bits are TxThresh, Lower are
                          * RxThreshold.  Irq can be asserted
                          * when rx fifo> thresh, txfifo<thresh
                  */
    cyg_uint8   prog_out; /* Set value of DTR (Bit0), RTS (Bit1)
                          *  if these bits are also enabled to GPIO_o
                  */

    cyg_uint8   unused1;
    cyg_uint8   DeltaIPEdgeNoSense;    /* Low 4-bits, set corr bit to 1 to
                                        * detect irq on rising AND falling
                                        * edges for corresponding GPIO_i
                                        * if enabled (edge insensitive)
                  */
    cyg_uint8   DeltaIPConfig_Mask;    /* Upper 4 bits: 1 for posedge sense
                                        * 0 for negedge sense if
                                        * not configured for edge
                                        * insensitive (see above)
                                        * Lower 4 bits: Mask to enable change
                                        *  detection IRQ for corresponding
                                        *  GPIO_i
                      */
    cyg_uint8   DeltaIP_SyncIP;        /* Upper 4 bits show which bits
                      *  have changed (may set IRQ).
                      *  read automatically clears bit
                      * Lower 4 bits are actual status
                      */

    cyg_uint16  intEnableMask;    /* Same Bit defs for Mask and status */
    cyg_uint16  intStatus;
        #define DELTAIP     0x0001
        #define TXUNDERR    0x0002
        #define TXOVFERR    0x0004
        #define TXFIFOTHOLD 0x0008
        #define TXREADLATCH 0x0010
        #define TXFIFOEMT   0x0020
        #define RXUNDERR    0x0040
        #define RXOVFERR    0x0080
        #define RXTIMEOUT   0x0100
        #define RXFIFOFULL  0x0200
        #define RXFIFOTHOLD 0x0400
        #define RXFIFONE    0x0800
        #define RXFRAMERR   0x1000
        #define RXPARERR    0x2000
        #define RXBRK       0x4000

    cyg_uint16  unused2;
    cyg_uint16  Data;       /* Write to TX, Read from RX */
                            /* bits 11:8 are BRK,PAR,FRM errors */

    cyg_uint32  unused3;
    cyg_uint32  unused4;
} Uart;

// ----------------------------------------------------------------------------
// Interrupt Controller registers.  Also copied from bcm_map.h.
// ----------------------------------------------------------------------------

typedef struct IntControl {
    cyg_uint32  RevID;      /* (00) */
        #define SECMOD_EN       0x400
        #define DES3_EN         0x200
        #define DES56_EN        0x100
    cyg_uint16  testControl1;   /* (04) */
    cyg_uint16  blkEnables; /* (06) */
        #define TDM_CLK_EN      0x0800
        #define SEC_CLK_EN      0x0400
        #define USB_CLK_EN      0x0100
        #define EMAC_CLK_EN     0x0080
        #define TRC_MCLK_EN     0x0040
        #define CMAC_TRC_CLK_EN 0x0020
        #define CMAC_CLK_EN     0x0010
        #define UART_CLK_EN     0x0008
        #define EBI_CLK_EN      0x0004
        #define BUS_CLK_EN      0x0002
        #define CPU_CLK_EN      0x0001

    cyg_uint16  pll_control;    /* (08) */
                /* Bits [15:12] micro_pll: default 6
                 * Bits [7:4]   usb_pll : default 7
                 * Bits [2:0]   usb_pll_div : default 0
                 */
    cyg_uint8   diagControl;    /* (0a) */
                /* Bits [6:4] diag12
                 * Bits [2:0] diag16
                 */
    cyg_uint8   control;    /* (0b) */
        #define SOFT_RESET      0x10
        #define QUIET_XTAL      0x08
        #define USPHY_BYP       0x04
        #define DSPHY_BYP       0x02
        #define USBPHY_BYP      0x01

    cyg_uint32  IrqEnableMask;    /* (0c) */
        #define UART0IRQ        0x00000004
        #define UART1IRQ        0x00000002
        #define TIMRIRQ         0x00000001
    cyg_uint32  IrqStatus;      /* (10) */
    cyg_uint8   unused3;        /* (14) */

    /* Upper 4 bits RSVD, Lower 4 bits set disable edge sensitivity */
    cyg_uint8   ExtEdgeInSense;

    /* Upper 4 bits enable corresponding ExtIrq, Lower 4 bits clr IRQ */
    cyg_uint8   extIrqMskandDetClr;

    /* Upper 4 bits are current status of input, lower bits define edge
    * sensitivity.  If high, IRQ on rising edge. If low, IRQ on falling edge
    */
    cyg_uint8   extIrqStatEdgeConfig;
    cyg_uint8   TBusSel;                    /* (18) */
    cyg_uint8   unused4;                    /* (19) */
    cyg_uint8   ephyTest;                   /* (1a) */
        #define EPHY_TESTEN             0x40
        #define EPHY_EXT_TEST_MODE      0x3e
        #define EPHY_CLOCK_RESET        (0x1b<<1)
        #define EPHY_RESET_N            0x01

    cyg_uint8   miscControl;                /* (1b) */
        #define DIAG16_EPHY         0x20
        #define DIAG16_US           0x40
        #define DIAG16_DS           0x60
        #define DIAG16_TRC_USB      0x80
        #define DIAG16_ADA_3310     0xa0
        #define DIAG16_MAC          0xc0

        #define TP_IN_EN            0x08    /* enable TP_IN[9:0] as inputs */
        #define USPHY_BYP           0x04
        #define DSPHY_BYP           0x02
        #define USBPHY_BYP          0x01

    cyg_uint32  diag_in;          /* read diag input bus */
    cyg_uint32  testControl2;     /* (20) */
        #define HFCMAC_BIST_EN          0x00020000
        #define SPI_MASTER_BIST_EN      0x00010000
        #define PERIPH_BIST_EN          0x00008000
        #define USB_BIST_EN             0x00004000
        #define EMAC_BIST_EN            0x00002000
        #define EBI_BIST_EN             0x00001000
        #define MBIST_HOLD_L            0x00000800
        #define MBIST_DBG_EN            0x00000400
        #define MBIST_DMP_EN            0x00000200
        #define ITAG_BIST_SEL           0x00000100
        #define ITAG_BIST_EN            0x00000080
        #define IDATA_BIST_SEL          0x00000040
        #define IDATA_BIST_EN           0x00000020
        #define DTAG_BIST_SEL           0x00000010
        #define DTAG_BIST_EN            0x00000008
        #define DDATA_BIST_SEL          0x00000004
        #define DDATA_BIST_EN           0x00000002
        #define TLB_BIST_EN             0x00000001

} IntControl;

#define TIMER_IRQ_BIT       0
#define UART1_IRQ_BIT       1
#define UART0_IRQ_BIT       2
#define DOWNSTREAM_IRQ_BIT  3
#define CMAC_IRQ_BIT        4
#define USB_IRQ_BIT         5
#define EMAC_IRQ_BIT        8
#define EPHY_IRQ_BIT        12
#define DMA1_IRQ_BIT        14
#define DMA2_IRQ_BIT        15
#define DMA13_IRQ_BIT       26
#define DMA14_IRQ_BIT       27
#define DMA15_IRQ_BIT       28
#define DMA16_IRQ_BIT       29
#define DMA17_IRQ_BIT       30
#define DMA18_IRQ_BIT       31

// ----------------------------------------------------------------------------
// Timer control registers.
// ----------------------------------------------------------------------------
typedef struct Timer {
    cyg_uint16	unused0;
    cyg_uint8	TimerMask;
    cyg_uint8	TimerInts;
        #define	TIMER0		0x01
        #define	TIMER1		0x02
        #define	TIMER2		0x04
        #define	WATCHDOG	0x08
        #define	TIMER0EN	0x01
        #define	TIMER1EN	0x02
        #define	TIMER2EN	0x04
    cyg_uint32	TimerCtl0;
    cyg_uint32	TimerCtl1;
    cyg_uint32	TimerCtl2;
        #define	TIMERENABLE	0x80000000
        #define	RSTCNTCLR	0x40000000	
    cyg_uint32	TimerCnt0;
    cyg_uint32	TimerCnt1;
    cyg_uint32	TimerCnt2;
    cyg_uint32	WatchDogDefCount;

    /* Write 0xff00 0x00ff to Start timer
    * Write 0xee00 0x00ee to Stop and re-load default count
    * Read from this register returns current watch dog count
    */
    cyg_uint32	WatchDogCtl;

    /* Number of ticks for WD Reset pulse to last */
    cyg_uint32	WDResetCount;
} Timer;


// ----------------------------------------------------------------------------
// SDRAM Controller registers.  Also copied from bcm_map.h.
// ----------------------------------------------------------------------------

typedef struct Sdram
{
    cyg_uint16  unused1;
    cyg_uint16  initControl;  /* 02 */
        #define SDR_9BIT_COL        0x800   /* SDRAM utilizes 9-bit column address */
        #define SDR_32BIT           0x400   /* memory data path is 32 bits wide */
        #define SD_POWER_DOWN       0x200   /* put sdram into power down */
        #define SD_SELF_REFRESH     0x100   /* enable self refresh mode */
        #define SD_SOFT_RESET       0x080   /* soft reset all sdram controller regs */
        #define SDR_256MEG          0x030   /* SDRAM device size is 256 Mbits */
        #define SDR_128MEG          0x020   /* SDRAM device size is 128 Mbits */
        #define SDR_64MEG           0x010   /* SDRAM device size is 64 Mbits */
        #define SDR_16MEG           0x000   /* SDRAM device size is 16 Mbits */
        #define SD_MASTER_ENABLE    0x008   /* enable accesses to external sdram */
        #define SD_MRS              0x004   /* generate a mode register select cycle */
        #define SD_PRECHARGE        0x002   /* generate a precharge cycle */
        #define SD_CBR              0x001   /* generate a refresh cycle */
    cyg_uint8   unused2[3];
    cyg_uint8   config;        /* 07 */
        #define SD_FAST_MEM         0x04    /* 1=CAS latency of 2, 0 = CAS latency of 3 */
        #define SD_BURST_LEN        0x03    /* set burst length */
        #define SD_BURST_FULL_PAGE  0x00    /* .. full page */
        #define SD_BURST_8          0x01    /* .. 8 words */
        #define SD_BURST_4          0x02    /* .. 4 words */
        #define SD_BURST_2          0x03    /* .. 2 words */
    cyg_uint16  unused3;
    cyg_uint16  refreshControl; /* 0a */
        #define SD_REFRESH_ENABLE   0x8000  /* refresh enable */
        #define SD_REFRESH_PERIOD   0x00ff  /* refresh period (16 x n x clock_period) */

    cyg_uint32  memoryBase; /* 0c */
        #define SD_MEMBASE_MASK     0xffffe000  /* base address mask */
        #define SD_MEMSIZE_MASK     0x00000007  /* memory size mask */
        #define SD_MEMSIZE_2MEG     0x00000000  /* memory is 2 meg */
        #define SD_MEMSIZE_8MEG     0x00000001  /* memory is 8 meg */
        #define SD_MEMSIZE_16MEG    0x00000002  /* memory is 16 meg */
        #define SD_MEMSIZE_32MEG    0x00000003  /* memory is 32 meg */
        #define SD_MEMSIZE_64MEG    0x00000004  /* memory is 64 meg */

} Sdram;

// ----------------------------------------------------------------------------
// Define locations for registers
// ----------------------------------------------------------------------------

extern volatile IntControl *INTC;
extern volatile Uart       *UART;
extern volatile Uart       *UART1;
extern volatile Sdram      *SDRAM;
extern volatile Timer      *TIMER;

#define BCM_INTC_BASE           0xfffe0000  // interrupt controller registers
// Some BCM33xx chips have dual UARTs, but we're only using one.
#define BCM_UART0_BASE          0xfffe0300  // UART0 registers
#define BCM_UART1_BASE          0xfffe0320  // UART1 registers
#define BCM_SDRAM_BASE          0xfffe2300  // SDRAM control registers
#define BCM_TIMER_BASE          0xfffe0200  // Timer control registers

#endif // !defined( BCM33XX_REGS_H )
